#!/bin/bash
#

# build_genie.sh

# if this build script complains that it cannot find bzlib.h,
# you need to install bzip2-devel on Scientific Linux

usage()
{
   echo "USAGE: `basename ${0}` <product_directory> <e17|e19|c2|c7>[:py2] <debug|prof> [tar]"
}

# -------------------------------------------------------------------
# shared boilerplate
# -------------------------------------------------------------------

get_this_dir()
{
    ( cd / ; /bin/pwd -P ) >/dev/null 2>&1
    if (( $? == 0 )); then
      pwd_P_arg="-P"
    fi
    reldir=`dirname ${0}`
    thisdir=`cd ${reldir} && /bin/pwd ${pwd_P_arg}`
}

get_ssibuildshims()
{
    # make sure we can use the setup alias
    if [ -z ${UPS_DIR} ]
    then
       echo "ERROR: please setup ups"
       exit 1
    fi
    source `${UPS_DIR}/bin/ups setup ${SETUP_UPS}`

    PRODUCTS=${product_dir}:${PRODUCTS}

    setup ssibuildshims ${ssibuildshims_version}
    if [ -z ${SSIBUILDSHIMS_DIR} ]
    then
       echo "ERROR: failed to setup ssibuildshims ${ssibuildshims_version}"
       exit 1
    fi
}

# -------------------------------------------------------------------
# start processing
# -------------------------------------------------------------------

product_dir=${1}
basequal=${2}
extraqual=${3}
maketar=${4}

if [ -z ${product_dir} ]
then
   echo "ERROR: please specify the local product directory"
   usage
   exit 1
fi

# -------------------------------------------------------------------
# package name and version
# -------------------------------------------------------------------

package=genie
origpkgver=v3_00_06
pkgver=${origpkgver}f
pkgdotver=`echo ${origpkgver} | sed -e 's/_/./g' | sed -e 's/^v//'`
ssibuildshims_version=v1_04_14
geniedot=`echo ${origpkgver} | sed -e 's/_b/b/g' | sed -e 's/_/./g'`
geniever=`echo ${origpkgver} | sed -e 's/_b/b/' | sed -e 's/^v/R-/g'`
reweightver=R-1_00_04
geniedir=GENIE-Generator
reweightdir=GENIE-Reweight
#pkgtarname=GENIE-Generator_${geniedot}

get_this_dir

get_ssibuildshims

source define_basics --

if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
   exit 0
fi

echo "building ${package} for ${OS}-${plat}-${qualdir} (flavor ${flvr})"

# -------------------------------------------------------------------
# package dependent stuff goes here
# -------------------------------------------------------------------

if [ "${extraqual}" = "debug" ]
then
    extracommand="--enable-debug --with-optimiz-level=O0"
    #cxxflg="-g -O0"
    # compile flags for --enable-debug --with-optimiz-level=O0: g++ -c  -Wall -fPIC -g -O0   -Wno-strict-aliasing -ffriend-injection
elif [ "${extraqual}" = "prof" ]
then
    # enable-profiler needs path to profiler library, but we do not build against any profiler
    #extracommand="--enable-profiler --with-profiler-lib= --with-optimiz-level=O3"
    extracommand="--with-optimiz-level=O3"
    cxxflg="-g -DNDEBUG -fno-omit-frame-pointer"
    # compile flags for optimiz-level=O3: g++ -c  -Wall -fPIC  -O3   -Wno-strict-aliasing -ffriend-injection
fi

gcc_compilers="e1[79](:py2)?"
clang_compilers="c[27](:py2)?"
if [[ "${basequal}" =~ $gcc_compilers ]]
then
  cc=gcc
  cxx=g++
  cxxflg="${cxxflg} -std=c++17 -Wno-deprecated-declarations"
elif [[ "${basequal}" =~ $clang_compilers ]]
then
  cc=clang
  cxx=clang++
  ##cxxflg="${cxxflg} -std=c++17 -Wno-deprecated-declarations -D_LIBCPP_ENABLE_CXX17_REMOVED_FEATURES=1"
  cxxflg="${cxxflg} -std=c++17 -Wno-deprecated-declarations -Wno-register"
else
  ssi_die "Qualifier $basequal not recognized."
fi

echo "Building ${package} in ${pkgdir}"

mkdir -p ${pkgdir}
if [ ! -d ${pkgdir} ]
then
   echo "ERROR: failed to create ${pkgdir}"
   exit 1
fi

# fake ups declare
fakedb=${product_dir}/${package}/${pkgver}/fakedb
${SSIBUILDSHIMS_DIR}/bin/fake_declare_product ${product_dir} ${package} ${pkgver} ${fullqual}

setup -B ${package} ${pkgver} -q ${fullqual} -z ${fakedb}:${PRODUCTS} || ssi_die "fake setup failed"

if [ -z ${GENIE} ]
then
   echo "ERROR: failed to setup genie"
   exit 1
else
   echo "GENIE: ${GENIE}"
   echo "GENIE_REWEIGHT: ${GENIE_REWEIGHT}"
fi

if [ -z ${PYLIB} ]
then
   echo "ERROR: failed to setup pythia"
   exit 1
else
   echo "PYLIB: ${PYLIB}"
fi

if [ -z ${ROOTSYS} ]
then
   echo "ERROR: failed to setup root"
   exit 1
else
   echo "ROOTSYS: ${ROOTSYS}"
fi

if [ -z ${LIBXML2_INC} ]
then
   echo "ERROR: failed to setup libxml"
   exit 1
else
   echo "LIBXML2_INC: ${LIBXML2_INC}"
fi

if [ -z ${LOG4CPP_INC} ]
then
   echo "ERROR: failed to setup log4cpp"
   exit 1
else
   echo "LOG4CPP_INC: ${LOG4CPP_INC}"
fi

if [ -z ${LHAPDF_INC} ]
then
   echo "ERROR: failed to setup lhapdf"
   exit 1
else
   echo "LHAPDF_INC: ${LHAPDF_INC}"
fi

set -x

cd ${pkgdir}
tar -xf ${tardir}/${geniedir}.tar || ssi_die "failed.to unwind ${geniedir}.tar"
tar -xf ${tardir}/${reweightdir}.tar || ssi_die "failed.to unwind ${reweightdir}.tar"

# Remove unnecessary LHAPDF5 dependency; this is a temporary patch and
# is not necessary for future GENIE versions.
patch -b -p0 < ${patchdir}/remove_lhapdf5_dep.patch || ssi_die "Unable to apply ${pathdir}/remove_lhapdf5_dep.patch."

# fix a build problem
patch -b -p0 < ${patchdir}/${geniedir}.patch || ssi_die "Unable to apply ${patchdir}/${geniedir}.patch."
if [ ${OS1} = "Darwin" ];then
  patch -b -p0 < ${patchdir}/macos.patch || ssi_die "Unable to apply ${patchdir}/macos.patch."
fi

# build genie generator
cd ${GENIE} || ssi_die "cannot find ${GENIE}"

# configure generates .../src/make/Make.config
./configure  --prefix=${pkgdir} ${extracommand} \
--with-compiler=${cc} \
--disable-lhapdf5 \
--enable-fnal \
--enable-atmo \
--enable-nucleon-decay \
--enable-nnbar-oscillation \
--enable-boosted-dark-matter \
--with-pythia6-lib=${PYLIB} \
--with-libxml2-inc=${LIBXML2_INC} \
--with-libxml2-lib=${LIBXML2_FQ_DIR}/lib \
--with-log4cpp-inc=${LOG4CPP_INC} \
--with-log4cpp-lib=${LOG4CPP_FQ_DIR}/lib || ssi_die "configure failed."

# you must pass extra compiler flags directly to make
make GOPT_WITH_CXX_USERDEF_FLAGS="${cxxflg}" || ssi_die "make failed."
make install || ssi_die "make install failed."

# build genie reweight
cd ${GENIE_REWEIGHT} || ssi_die "cannot find ${GENIE_REWEIGHT}"

# you must pass extra compiler flags directly to make
make GOPT_WITH_CXX_USERDEF_FLAGS="${cxxflg}" || ssi_die "make failed."
make install || ssi_die "make install failed."

#clean now
cd ${GENIE} || ssi_die "cannot find ${GENIE}"
make clean || ssi_die "make clean failed."
cd ${GENIE_REWEIGHT} || ssi_die "cannot find ${GENIE_REWEIGHT}"
make clean || ssi_die "make clean failed."


set +x

if [ ! -d ${pkgdir}/lib ]
then
   echo "ERROR: failed to create ${pkgdir}/lib"
   echo "       Something is very wrong"
   exit 1
fi

# real ups declare
${SSIBUILDSHIMS_DIR}/bin/declare_product ${product_dir} ${package} ${pkgver} ${fullqual}

# -------------------------------------------------------------------
# common bottom stuff
# -------------------------------------------------------------------

# this should not complain
echo "Finished building ${package} ${pkgver}"
setup -B ${package} ${pkgver} -q ${fullqual} -z ${product_dir} || ssi_die "setup failed"
echo "${package} is installed at ${GENIE}"

if [ ${OS1} = "Darwin" ];then
  echo
  echo "fix library rpaths"
  echo
  cd ${GENIE_FQ_DIR}/lib
  fix_macos_lib_symbols "*${pkgdotver}.dylib"
  cd ${thisdir}
fi

echo ""
echo "FIX PATHS in ${GENIE}/src/make/Make.config"
echo ""

filename=/tmp/geniefix-$$$$.sh

if [ -e ${filename} ]; then rm -f ${filename}; fi

echo "#!/bin/bash" > ${filename}
echo " set -x " >>  ${filename}
echo "mv ${GENIE}/src/make/Make.config ${GENIE}/src/make/Make.config.bak" >> ${filename}
echo "cat ${GENIE}/src/make/Make.config.bak | sed 's%${PYLIB}%\${PYLIB}%' | sed 's%${GENIE_FQ_DIR}%\${GENIE_FQ_DIR}%'  | sed 's%${LHAPDF_FQ_DIR}%\${LHAPDF_FQ_DIR}%' | sed 's%${LOG4CPP_FQ_DIR}%\${LOG4CPP_FQ_DIR}%' | sed 's%${LIBXML2_FQ_DIR}%\${LIBXML2_FQ_DIR}%'> ${GENIE}/src/make/Make.config" >> ${filename}
echo " set +x " >>  ${filename}

chmod +x ${filename}
${filename}

if [ -e ${filename} ]; then rm -f ${filename}; fi

# this must be last
if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
fi

exit 0
